@@ -1,6 +1,7 @@
 #include <string>
 #include <algorithm>
 #include <QMessageBox>
+#include "qt-wrappers.hpp"
 #include "audio-encoders.hpp"
 #include "window-basic-main.hpp"
 #include "window-basic-main-outputs.hpp"
@@ -47,11 +48,14 @@ static void OBSStopStreaming(void *data, calldata_t *params)
 {
 	BasicOutputHandler *output = static_cast<BasicOutputHandler*>(data);
 	int code = (int)calldata_int(params, "code");
+	const char *last_error = calldata_string(params, "last_error");
+
+	QString arg_last_error = QString::fromUtf8(last_error);
 
 	output->streamingActive = false;
 	output->delayActive = false;
 	QMetaObject::invokeMethod(output->main,
-			"StreamingStop", Q_ARG(int, code));
+			"StreamingStop", Q_ARG(int, code), Q_ARG(QString, arg_last_error));
 }
 
 static void OBSStartRecording(void *data, calldata_t *params)
@@ -565,21 +569,21 @@ void SimpleOutput::UpdateStreamingSettings_amd(obs_data_t *settings,
 		int bitrate)
 {
 	// Static Properties
-	obs_data_set_int(settings, "AMF.H264.Usage", 0);
-	obs_data_set_int(settings, "AMF.H264.Profile", 100); // High
+	obs_data_set_int(settings, "Usage", 0);
+	obs_data_set_int(settings, "Profile", 100); // High
 	obs_data_set_string(settings, "profile", "high"); // High
 	
 	// Rate Control Properties
-	obs_data_set_int(settings, "AMF.H264.RateControlMethod", 1);
+	obs_data_set_int(settings, "RateControlMethod", 1);
 	obs_data_set_string(settings, "rate_control", "CBR");
-	obs_data_set_int(settings, "AMF.H264.Bitrate.Target", bitrate);
+	obs_data_set_int(settings, "Bitrate.Target", bitrate);
 	obs_data_set_int(settings, "bitrate", bitrate);
-	obs_data_set_int(settings, "AMF.H264.FillerData", 1);
-	obs_data_set_int(settings, "AMF.H264.VBVBuffer", 1);
-	obs_data_set_int(settings, "AMF.H264.VBVBuffer.Size", bitrate);
+	obs_data_set_int(settings, "FillerData", 1);
+	obs_data_set_int(settings, "VBVBuffer", 1);
+	obs_data_set_int(settings, "VBVBuffer.Size", bitrate);
 	
 	// Picture Control Properties
-	obs_data_set_double(settings, "AMF.H264.KeyframeInterval", 2.0);
+	obs_data_set_double(settings, "KeyframeInterval", 2.0);
 	obs_data_set_int(settings, "keyint_sec", 2);
 }
 
@@ -588,21 +592,21 @@ void SimpleOutput::UpdateRecordingSettings_amd_cqp(int cqp)
 	obs_data_t *settings = obs_data_create();
 
 	// Static Properties
-	obs_data_set_int(settings, "AMF.H264.Usage", 0);
-	obs_data_set_int(settings, "AMF.H264.Profile", 100); // High
+	obs_data_set_int(settings, "Usage", 0);
+	obs_data_set_int(settings, "Profile", 100); // High
 	obs_data_set_string(settings, "profile", "high"); // High
 
 	// Rate Control Properties
-	obs_data_set_int(settings, "AMF.H264.RateControlMethod", 0);
+	obs_data_set_int(settings, "RateControlMethod", 0);
 	obs_data_set_string(settings, "rate_control", "CQP");
-	obs_data_set_int(settings, "AMF.H264.QP.IFrame", cqp);
-	obs_data_set_int(settings, "AMF.H264.QP.PFrame", cqp);
-	obs_data_set_int(settings, "AMF.H264.QP.BFrame", cqp);
-	obs_data_set_int(settings, "AMF.H264.VBVBuffer", 1);
-	obs_data_set_int(settings, "AMF.H264.VBVBuffer.Size", 50000);
+	obs_data_set_int(settings, "QP.IFrame", cqp);
+	obs_data_set_int(settings, "QP.PFrame", cqp);
+	obs_data_set_int(settings, "QP.BFrame", cqp);
+	obs_data_set_int(settings, "VBVBuffer", 1);
+	obs_data_set_int(settings, "VBVBuffer.Size", 100000);
 
 	// Picture Control Properties
-	obs_data_set_double(settings, "AMF.H264.KeyframeInterval", 2.0);
+	obs_data_set_double(settings, "KeyframeInterval", 2.0);
 	obs_data_set_int(settings, "keyint_sec", 2);
 
 	// Update and release
@@ -778,7 +782,7 @@ bool SimpleOutput::ConfigureRecording(bool updateReplayBuffer)
 
 	if (!dir) {
 		if (main->isVisible())
-			QMessageBox::information(main,
+			OBSMessageBox::information(main,
 					QTStr("Output.BadPath.Title"),
 					QTStr("Output.BadPath.Text"));
 		else
@@ -1398,7 +1402,7 @@ bool AdvancedOutput::StartRecording()
 
 		if (!dir) {
 			if (main->isVisible())
-				QMessageBox::information(main,
+				OBSMessageBox::information(main,
 						QTStr("Output.BadPath.Title"),
 						QTStr("Output.BadPath.Text"));
 			else
