@@ -19,13 +19,14 @@ private:
 	void mousePressEvent(QMouseEvent *event) override;
 
 	int savedMonitor = 0;
+	bool isWindow = false;
 
 private slots:
 	void EscapeTriggered();
 
 public:
-	OBSProjector(QWidget *parent, obs_source_t *source);
+	OBSProjector(QWidget *parent, obs_source_t *source, bool window);
 	~OBSProjector();
 
-	void Init(int monitor);
+	void Init(int monitor, bool window, QString title);
 };
