@@ -170,7 +170,7 @@ static void *aac_create(obs_data_t *settings, obs_encoder_t *encoder)
 		enc->context->cutoff = cutoff;
 	}
 
-	info("bitrate: %d, channels: %d",
+	info("bitrate: %" PRId64 ", channels: %d",
 			enc->context->bit_rate / 1000, enc->context->channels);
 
 	init_sizes(enc, audio);
@@ -259,7 +259,7 @@ static obs_properties_t *aac_properties(void *unused)
 	obs_properties_t *props = obs_properties_create();
 
 	obs_properties_add_int(props, "bitrate",
-			obs_module_text("Bitrate"), 32, 320, 32);
+			obs_module_text("Bitrate"), 64, 320, 32);
 	return props;
 }
 
